@@ -2,16 +2,18 @@
 
 from datetime import datetime
 from os.path import join as path
-from typing import Tuple
+from typing import List, Union, Tuple
 
-from .models import Group, Entity, State, Domain, JsonModel
+from .models import Group, Entity, State, Domain, JsonModel, Event
 from .errors import APIConfigurationError, ResponseError
 from .rawapi import RawWrapper
 
 
+FMT = '%Y-%m-%dT%H:%M:%S%z'
+
+
 class RawClient(RawWrapper):
     """The class used to interact with all of the api"""
-
     def __init__(self, *args, **kwargs) -> None:
         """Passes API authentication data to RawWrapper and validates API status"""
         super().__init__(*args, **kwargs)
@@ -19,7 +21,6 @@ class RawClient(RawWrapper):
         self.check_api_config()
 
     def __repr__(self) -> str:
-        """Returns string to intuitively indentify Client classes from `repr()`"""
         return f'<Client of "{self.api_url[:20]}">'
 
     # Response processing methods
@@ -32,13 +33,17 @@ class RawClient(RawWrapper):
         return domain
 
     def process_state_json(self, json: dict) -> State:
-        """Constructs State models from json data"""
+        """Constructs State model from json data"""
         return State(**json)
 
+    def process_event_json(self, json: dict) -> Event:
+        """Constructs Event model from json data"""
+        return Event(**json)
+
     # API information methods
     def api_error_log(self) -> str:
         """Returns the server error log as a string"""
-        return self.request('error_log', return_text_if_fail=True)
+        return self.request('error_log', return_text=True)
 
     def api_config(self) -> dict:
         """Returns the yaml configuration of homeassistant"""
@@ -47,25 +52,53 @@ class RawClient(RawWrapper):
 
     def logbook_entries(
         self,
-        entity: Entity = None,
-        start_date: datetime = None,  # Defaults to 1 day before
-        end_date: datetime = None
-    ):
-        raise NotImplementedError('Not implemented in this pre-release as of 2.0a1')
+        filter_entity: Entity = None,
+        timestamp: Union[str, datetime] = None,  # Defaults to 1 day before
+        end_timestamp: Union[str, datetime] = None
+    ) -> List[dict]:
+        params = {}
+        if filter_entity is not None:
+            params.update(entity=filter_entity.entity_id)
+        if end_timestamp is not None:
+            if isinstance(end_timestamp, datetime):
+                end_timestamp = end_timestamp.strftime(FMT)
+            params.update(end_time=end_timestamp)
+        if timestamp is not None:
+            if isinstance(timestamp, datetime):
+                timestamp = timestamp.strftime(FMT)
+            url = path('logbook', timestamp)
+        else:
+            url = 'logbook'
+        return self.request(url, params=params)
 
     def get_history(
         self,
-        entities: tuple = None,
-        entity: Entity = None,
-        start_date: datetime = None,  # Defaults to 1 day before
-        end_date: datetime = None,
+        entities: Tuple[Entity] = None,
+        timestamp: datetime = None,  # Defaults to 1 day before
+        end_timestamp: datetime = None,
         minimal_state_data=False,
-        big_changes_only=False
-    ):
-        raise NotImplementedError('Not implemented in this pre-release as of 2.0a1')
+        significant_changes_only=False
+    ) -> Tuple[Tuple[dict]]:
+        params = {}
+        if entities is not None:
+            params.update(filter_entity_id=','.join([ent.entity_id for ent in entities]))
+        if end_timestamp:
+            end_timestamp = end_timestamp.strftime(FMT)
+            params.update(end_time=end_timestamp)
+        if minimal_state_data:
+            params.update(minimal_response=None)
+        if significant_changes_only:
+            params.update(significant_changes_only=None)
+        if timestamp is not None:
+            if isinstance(timestamp, datetime):
+                timestamp = timestamp.strftime(FMT)
+            url = path('history/period', timestamp)
+        else:
+            url = 'history/period'
+        return self.request(url, params=self.construct_params(params))
 
-    def get_rendered_template(self, template: str):
-        raise NotImplementedError('Not implemented in this pre-release as of 2.0a1')
+    def get_rendered_template(self, template: str) -> str:
+        return self.request('template', json=dict(template=template), return_text=True, method='POST')
 
     def get_discovery_info(self) -> dict:
         """Returns a dictionary of discovery info such as internal_url and version"""
@@ -147,7 +180,7 @@ class RawClient(RawWrapper):
         domain: str,
         service: str,
         **service_data
-    ) -> Tuple[State]:
+    ) -> List[State]:
         """Tells homeassistant to trigger a service, returns stats changed while being called"""
         data = self.request(
             path(
@@ -192,17 +225,30 @@ class RawClient(RawWrapper):
         )
         return self.process_state_json(data)
 
-    def get_states(self) -> Tuple[State]:
+    def get_states(self) -> List[State]:
         """Gets the states of all entitites within homeassistant"""
         data = self.request('states')
         return [self.process_state_json(state_data) for state_data in data]
 
     # Event methods
-    def get_events(self):
-        raise NotImplementedError('Not implemented in this pre-release as of 2.0a1')
+    def get_events(self) -> JsonModel:
+        """Gets the Events that happen within homeassistant"""
+        data = self.request('events')
+        events = [self.process_event_json(event_info) for event_info in data]
+        events = {evt.event_type: evt for evt in events}
+        return JsonModel(events)
 
-    def fire_event(self):
-        raise NotImplementedError('Not implemented in this pre-release as of 2.0a1')
+    def fire_event(self, event_type: str, **event_data) -> str:
+        """Fires a given event_type within homeassistant. Must be an existing event_type."""
+        data = self.request(
+            path(
+                'events',
+                event_type
+            ),
+            method='POST',
+            json=event_data
+        )
+        return data.get('message', 'No message provided')
 
 
 class Client(RawClient):
